//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/FTDecayFunctionItemCatalogs.h
//! @brief     Defines FTDecayFunctionItemCatalog classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_FTDECAYFUNCTIONITEMCATALOGS_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_FTDECAYFUNCTIONITEMCATALOGS_H

#include <QString>
#include <QVector>

class FTDecayFunction1DItem;
class FTDecayFunction2DItem;

class FTDecayFunction1DItemCatalog {
public:
    using CatalogedType = FTDecayFunction1DItem;

    // Do not change the numbering! It is serialized!
    enum class Type : uint8_t { Cauchy = 1, Gauss = 2, Triangle = 3, Voigt = 4 };

    struct UiInfo {
        QString menuEntry;
        QString description;
        QString iconPath;
    };

    //! Creates the item of the given type.
    static CatalogedType* create(Type type);

    //! List of available types, sorted as expected in the UI.
    static QVector<Type> types();

    //! UiInfo on the given type.
    static UiInfo uiInfo(Type t);

    //! Returns the enum type of the given item.
    static Type type(CatalogedType* item);
};

class FTDecayFunction2DItemCatalog {
public:
    using CatalogedType = FTDecayFunction2DItem;

    // Do not change the numbering! It is serialized!
    enum class Type : uint8_t { Cauchy = 1, Gauss = 2, Voigt = 3 };

    struct UiInfo {
        QString menuEntry;
        QString description;
        QString iconPath;
    };

    //! Creates the item of the given type.
    static CatalogedType* create(Type type);

    //! List of available types, sorted as expected in the UI.
    static QVector<Type> types();

    //! UiInfo on the given type.
    static UiInfo uiInfo(Type t);

    //! Returns the enum type of the given item.
    static Type type(const CatalogedType* item);
};


#endif // BORNAGAIN_GUI_MODEL_SAMPLE_FTDECAYFUNCTIONITEMCATALOGS_H
