//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/ParticleItem.cpp
//! @brief     Implements class ParticleItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/ParticleItem.h"
#include "GUI/Model/Job/JobItem.h"
#include "GUI/Model/Material/MaterialItem.h"
#include "GUI/Model/Sample/FormFactorItemCatalog.h"
#include "GUI/Model/Sample/FormFactorItems.h"
#include "GUI/Model/Sample/ParticleCoreShellItem.h"
#include "GUI/Model/Sample/RotationItemCatalog.h"
#include "GUI/Model/Session/Serializer.h"
#include "Sample/Particle/Particle.h"

namespace {

const QString abundance_tooltip = "Proportion of this type of particles normalized to the \n"
                                  "total number of particles in the layout";

const QString position_tooltip = "Relative position of the particle's reference point \n"
                                 "in the coordinate system of the parent (nm)";

} // namespace

ParticleItem::ParticleItem() : ItemWithParticles(abundance_tooltip, position_tooltip)
{
    m_formFactor.init<FormFactorItemCatalog>("Form Factor", "", "formFactor");
    m_formFactor.set(new CylinderItem());
}

void ParticleItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw(m_abundance);
    s.rw(m_position);
    s.rw<RotationItemCatalog>(m_rotation);
    s.rw<FormFactorItemCatalog>(m_formFactor);
}

std::unique_ptr<Particle> ParticleItem::createParticle() const
{
    auto domainMaterial = materialItem()->createMaterial();

    auto particle = std::make_unique<Particle>(*domainMaterial, *m_formFactor->createFormFactor());
    particle->setAbundance(abundance());

    setTransformationInfo(particle.get());

    return particle;
}

void ParticleItem::setFormFactor(FormFactorItem* p)
{
    m_formFactor.set(p);
}

FormFactorItem* ParticleItem::formfactor() const
{
    return m_formFactor.get();
}

QVector<ItemWithParticles*> ParticleItem::containedItemsWithParticles() const
{
    return {};
}
