//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/RotationItems.h
//! @brief     Defines class RotationItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_ROTATIONITEMS_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_ROTATIONITEMS_H

#include "GUI/Model/Types/DoubleProperty.h"
#include <memory>

class IRotation;
class RotMatrix;
class Serializer;

using std::unique_ptr;

class RotationItem {
public:
    virtual ~RotationItem() = default;
    RotMatrix rotation() const;

    virtual void serialize(Serializer& s) = 0;

protected:
    RotationItem();
    virtual unique_ptr<IRotation> createRotation() const = 0;
};

class XRotationItem : public RotationItem {
public:
    XRotationItem();
    void serialize(Serializer& s) override;

    DoubleDescriptor angle() const;

protected:
    unique_ptr<IRotation> createRotation() const override;

private:
    DoubleProperty m_angle;
};

class YRotationItem : public RotationItem {
public:
    YRotationItem();
    void serialize(Serializer& s) override;

    DoubleDescriptor angle() const;

protected:
    unique_ptr<IRotation> createRotation() const override;

private:
    DoubleProperty m_angle;
};

class ZRotationItem : public RotationItem {
public:
    ZRotationItem();
    void serialize(Serializer& s) override;

    DoubleDescriptor angle() const;

protected:
    unique_ptr<IRotation> createRotation() const override;

private:
    DoubleProperty m_angle;
};

class EulerRotationItem : public RotationItem {
public:
    EulerRotationItem();
    void serialize(Serializer& s) override;

    DoubleDescriptor alpha() const;
    DoubleDescriptor beta() const;
    DoubleDescriptor gamma() const;

protected:
    unique_ptr<IRotation> createRotation() const override;

private:
    DoubleProperty m_alpha;
    DoubleProperty m_beta;
    DoubleProperty m_gamma;
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_ROTATIONITEMS_H
